/*
 Legal Notice: The source code contained in this file has been derived from
 the source code of Encryption for the Masses 2.02a, which is Copyright (c)
 Paul Le Roux and which is covered by the 'License Agreement for Encryption
 for the Masses'. Modifications and additions to that source code contained
 in this file are Copyright (c) TrueCrypt Foundation and are covered by the
 TrueCrypt License 2.2 the full text of which is contained in the file
 License.txt included in TrueCrypt binary and source code distribution
 packages. */

#include "Tcdefs.h"
#include "Crc.h"
#include "Crypto.h"
#include "Common/Endian.h"
#include "Tests.h"

#ifdef LINUX_DRIVER
#include <linux/string.h>
#else
#include <string.h>
#include "Pkcs5.h"
#endif

/* Blowfish Test Vectors */

/* Blowfish test vectors from www.counterpane.com/blowfish.html */

#define BF_TEST_COUNT 34

typedef struct {
	unsigned char key[8];
	unsigned char plaintext[8];
	unsigned char ciphertext[8];
	} BF_TEST;

BF_TEST bf_ecb_vectors[BF_TEST_COUNT] = {
"\x00\x00\x00\x00\x00\x00\x00\x00","\x00\x00\x00\x00\x00\x00\x00\x00","\x4E\xF9\x97\x45\x61\x98\xDD\x78",
"\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF","\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF","\x51\x86\x6F\xD5\xB8\x5E\xCB\x8A",
"\x30\x00\x00\x00\x00\x00\x00\x00","\x10\x00\x00\x00\x00\x00\x00\x01","\x7D\x85\x6F\x9A\x61\x30\x63\xF2",
"\x11\x11\x11\x11\x11\x11\x11\x11","\x11\x11\x11\x11\x11\x11\x11\x11","\x24\x66\xDD\x87\x8B\x96\x3C\x9D",
"\x01\x23\x45\x67\x89\xAB\xCD\xEF","\x11\x11\x11\x11\x11\x11\x11\x11","\x61\xF9\xC3\x80\x22\x81\xB0\x96",
"\x11\x11\x11\x11\x11\x11\x11\x11","\x01\x23\x45\x67\x89\xAB\xCD\xEF","\x7D\x0C\xC6\x30\xAF\xDA\x1E\xC7",
"\x00\x00\x00\x00\x00\x00\x00\x00","\x00\x00\x00\x00\x00\x00\x00\x00","\x4E\xF9\x97\x45\x61\x98\xDD\x78",
"\xFE\xDC\xBA\x98\x76\x54\x32\x10","\x01\x23\x45\x67\x89\xAB\xCD\xEF","\x0A\xCE\xAB\x0F\xC6\xA0\xA2\x8D",
"\x7C\xA1\x10\x45\x4A\x1A\x6E\x57","\x01\xA1\xD6\xD0\x39\x77\x67\x42","\x59\xC6\x82\x45\xEB\x05\x28\x2B",
"\x01\x31\xD9\x61\x9D\xC1\x37\x6E","\x5C\xD5\x4C\xA8\x3D\xEF\x57\xDA","\xB1\xB8\xCC\x0B\x25\x0F\x09\xA0",
"\x07\xA1\x13\x3E\x4A\x0B\x26\x86","\x02\x48\xD4\x38\x06\xF6\x71\x72","\x17\x30\xE5\x77\x8B\xEA\x1D\xA4",
"\x38\x49\x67\x4C\x26\x02\x31\x9E","\x51\x45\x4B\x58\x2D\xDF\x44\x0A","\xA2\x5E\x78\x56\xCF\x26\x51\xEB",
"\x04\xB9\x15\xBA\x43\xFE\xB5\xB6","\x42\xFD\x44\x30\x59\x57\x7F\xA2","\x35\x38\x82\xB1\x09\xCE\x8F\x1A",
"\x01\x13\xB9\x70\xFD\x34\xF2\xCE","\x05\x9B\x5E\x08\x51\xCF\x14\x3A","\x48\xF4\xD0\x88\x4C\x37\x99\x18",
"\x01\x70\xF1\x75\x46\x8F\xB5\xE6","\x07\x56\xD8\xE0\x77\x47\x61\xD2","\x43\x21\x93\xB7\x89\x51\xFC\x98",
"\x43\x29\x7F\xAD\x38\xE3\x73\xFE","\x76\x25\x14\xB8\x29\xBF\x48\x6A","\x13\xF0\x41\x54\xD6\x9D\x1A\xE5",
"\x07\xA7\x13\x70\x45\xDA\x2A\x16","\x3B\xDD\x11\x90\x49\x37\x28\x02","\x2E\xED\xDA\x93\xFF\xD3\x9C\x79",
"\x04\x68\x91\x04\xC2\xFD\x3B\x2F","\x26\x95\x5F\x68\x35\xAF\x60\x9A","\xD8\x87\xE0\x39\x3C\x2D\xA6\xE3",
"\x37\xD0\x6B\xB5\x16\xCB\x75\x46","\x16\x4D\x5E\x40\x4F\x27\x52\x32","\x5F\x99\xD0\x4F\x5B\x16\x39\x69",
"\x1F\x08\x26\x0D\x1A\xC2\x46\x5E","\x6B\x05\x6E\x18\x75\x9F\x5C\xCA","\x4A\x05\x7A\x3B\x24\xD3\x97\x7B",
"\x58\x40\x23\x64\x1A\xBA\x61\x76","\x00\x4B\xD6\xEF\x09\x17\x60\x62","\x45\x20\x31\xC1\xE4\xFA\xDA\x8E",
"\x02\x58\x16\x16\x46\x29\xB0\x07","\x48\x0D\x39\x00\x6E\xE7\x62\xF2","\x75\x55\xAE\x39\xF5\x9B\x87\xBD",
"\x49\x79\x3E\xBC\x79\xB3\x25\x8F","\x43\x75\x40\xC8\x69\x8F\x3C\xFA","\x53\xC5\x5F\x9C\xB4\x9F\xC0\x19",
"\x4F\xB0\x5E\x15\x15\xAB\x73\xA7","\x07\x2D\x43\xA0\x77\x07\x52\x92","\x7A\x8E\x7B\xFA\x93\x7E\x89\xA3",
"\x49\xE9\x5D\x6D\x4C\xA2\x29\xBF","\x02\xFE\x55\x77\x81\x17\xF1\x2A","\xCF\x9C\x5D\x7A\x49\x86\xAD\xB5",
"\x01\x83\x10\xDC\x40\x9B\x26\xD6","\x1D\x9D\x5C\x50\x18\xF7\x28\xC2","\xD1\xAB\xB2\x90\x65\x8B\xC7\x78",
"\x1C\x58\x7F\x1C\x13\x92\x4F\xEF","\x30\x55\x32\x28\x6D\x6F\x29\x5A","\x55\xCB\x37\x74\xD1\x3E\xF2\x01",
"\x01\x01\x01\x01\x01\x01\x01\x01","\x01\x23\x45\x67\x89\xAB\xCD\xEF","\xFA\x34\xEC\x48\x47\xB2\x68\xB2",
"\x1F\x1F\x1F\x1F\x0E\x0E\x0E\x0E","\x01\x23\x45\x67\x89\xAB\xCD\xEF","\xA7\x90\x79\x51\x08\xEA\x3C\xAE",
"\xE0\xFE\xE0\xFE\xF1\xFE\xF1\xFE","\x01\x23\x45\x67\x89\xAB\xCD\xEF","\xC3\x9E\x07\x2D\x9F\xAC\x63\x1D",
"\x00\x00\x00\x00\x00\x00\x00\x00","\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF","\x01\x49\x33\xE0\xCD\xAF\xF6\xE4",
"\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF","\x00\x00\x00\x00\x00\x00\x00\x00","\xF2\x1E\x9A\x77\xB7\x1C\x49\xBC",
"\x01\x23\x45\x67\x89\xAB\xCD\xEF","\x00\x00\x00\x00\x00\x00\x00\x00","\x24\x59\x46\x88\x57\x54\x36\x9A",
"\xFE\xDC\xBA\x98\x76\x54\x32\x10","\xFF\xFF\xFF\xFF\xFF\xFF\xFF\xFF","\x6B\x5C\x5A\x9C\x5D\x9E\x0A\x5A"
};


/* DES Test Vectors */

/* DES test vectors, derived from "Validating the Correctness of Hardware
   Implementations of the NBS Data Encryption Standard", NBS Special
   Publication 500-20, 1980 */

/* The data structure for the ( key, plaintext, ciphertext ) triplets */

typedef struct {
	unsigned char key[8];
	unsigned char plaintext[8];
	unsigned char ciphertext[8];
	} DES_TEST;


#define DES56_TEST_COUNT_IP 64
#define DES56_TEST_COUNT_VP 64
#define DES56_TEST_COUNT_KP 56
#define DES56_TEST_COUNT_RS 56
#define DES56_TEST_COUNT_DP 32
#define DES56_TEST_COUNT_SB 19

DES_TEST des56_ecb_vectors_ip[DES56_TEST_COUNT_IP] = {
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x95, 0xF8, 0xA5, 0xE5, 0xDD, 0x31, 0xD9, 0x00 },
	  { 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0xDD, 0x7F, 0x12, 0x1C, 0xA5, 0x01, 0x56, 0x19 },
	  { 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x2E, 0x86, 0x53, 0x10, 0x4F, 0x38, 0x34, 0xEA },
	  { 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x4B, 0xD3, 0x88, 0xFF, 0x6C, 0xD8, 0x1D, 0x4F },
	  { 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x20, 0xB9, 0xE7, 0x67, 0xB2, 0xFB, 0x14, 0x56 },
	  { 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x55, 0x57, 0x93, 0x80, 0xD7, 0x71, 0x38, 0xEF },
	  { 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x6C, 0xC5, 0xDE, 0xFA, 0xAF, 0x04, 0x51, 0x2F },
	  { 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x0D, 0x9F, 0x27, 0x9B, 0xA5, 0xD8, 0x72, 0x60 },
	  { 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0xD9, 0x03, 0x1B, 0x02, 0x71, 0xBD, 0x5A, 0x0A },
	  { 0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x42, 0x42, 0x50, 0xB3, 0x7C, 0x3D, 0xD9, 0x51 },
	  { 0x00, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0xB8, 0x06, 0x1B, 0x7E, 0xCD, 0x9A, 0x21, 0xE5 },
	  { 0x00, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0xF1, 0x5D, 0x0F, 0x28, 0x6B, 0x65, 0xBD, 0x28 },
	  { 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0xAD, 0xD0, 0xCC, 0x8D, 0x6E, 0x5D, 0xEB, 0xA1 },  
	  { 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xE6, 0xD5, 0xF8, 0x27, 0x52, 0xAD, 0x63, 0xD1 },  
	  { 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xEC, 0xBF, 0xE3, 0xBD, 0x3F, 0x59, 0x1A, 0x5E },  
	  { 0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xF3, 0x56, 0x83, 0x43, 0x79, 0xD1, 0x65, 0xCD },  
	  { 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x2B, 0x9F, 0x98, 0x2F, 0x20, 0x03, 0x7F, 0xA9 },
	  { 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x88, 0x9D, 0xE0, 0x68, 0xA1, 0x6F, 0x0B, 0xE6 },  
	  { 0x00, 0x00, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xE1, 0x9E, 0x27, 0x5D, 0x84, 0x6A, 0x12, 0x98 },  
	  { 0x00, 0x00, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x32, 0x9A, 0x8E, 0xD5, 0x23, 0xD7, 0x1A, 0xEC },
	  { 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xE7, 0xFC, 0xE2, 0x25, 0x57, 0xD2, 0x3C, 0x97 },  
	  { 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x12, 0xA9, 0xF5, 0x81, 0x7F, 0xF2, 0xD6, 0x5D },  
	  { 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0xA4, 0x84, 0xC3, 0xAD, 0x38, 0xDC, 0x9C, 0x19 },
	  { 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xFB, 0xE0, 0x0A, 0x8A, 0x1E, 0xF8, 0xAD, 0x72 },  
	  { 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x75, 0x0D, 0x07, 0x94, 0x07, 0x52, 0x13, 0x63 },  
	  { 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x64, 0xFE, 0xED, 0x9C, 0x72, 0x4C, 0x2F, 0xAF },  
	  { 0x00, 0x00, 0x00, 0x40, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xF0, 0x2B, 0x26, 0x3B, 0x32, 0x8E, 0x2B, 0x60 },  
	  { 0x00, 0x00, 0x00, 0x20, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x9D, 0x64, 0x55, 0x5A, 0x9A, 0x10, 0xB8, 0x52 },  
	  { 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0xD1, 0x06, 0xFF, 0x0B, 0xED, 0x52, 0x55, 0xD7 },  
	  { 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xE1, 0x65, 0x2C, 0x6B, 0x13, 0x8C, 0x64, 0xA5 },  
	  { 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xE4, 0x28, 0x58, 0x11, 0x86, 0xEC, 0x8F, 0x46 },  
	  { 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xAE, 0xB5, 0xF5, 0xED, 0xE2, 0x2D, 0x1A, 0x36 },  
	  { 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xE9, 0x43, 0xD7, 0x56, 0x8A, 0xEC, 0x0C, 0x5C },  
	  { 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xDF, 0x98, 0xC8, 0x27, 0x6F, 0x54, 0xB0, 0x4B },
	  { 0x00, 0x00, 0x00, 0x00, 0x40, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xB1, 0x60, 0xE4, 0x68, 0x0F, 0x6C, 0x69, 0x6F },  
	  { 0x00, 0x00, 0x00, 0x00, 0x20, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xFA, 0x07, 0x52, 0xB0, 0x7D, 0x9C, 0x4A, 0xB8 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xCA, 0x3A, 0x2B, 0x03, 0x6D, 0xBC, 0x85, 0x02 },
	  { 0x00, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x5E, 0x09, 0x05, 0x51, 0x7B, 0xB5, 0x9B, 0xCF },  
	  { 0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x81, 0x4E, 0xEB, 0x3B, 0x91, 0xD9, 0x07, 0x26 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x4D, 0x49, 0xDB, 0x15, 0x32, 0x91, 0x9C, 0x9F },
	  { 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x25, 0xEB, 0x5F, 0xC3, 0xF8, 0xCF, 0x06, 0x21 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xAB, 0x6A, 0x20, 0xC0, 0x62, 0x0D, 0x1C, 0x6F },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x79, 0xE9, 0x0D, 0xBC, 0x98, 0xF9, 0x2C, 0xCA },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x86, 0x6E, 0xCE, 0xDD, 0x80, 0x72, 0xBB, 0x0E },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x8B, 0x54, 0x53, 0x6F, 0x2F, 0x3E, 0x64, 0xA8 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0xEA, 0x51, 0xD3, 0x97, 0x55, 0x95, 0xB8, 0x6B },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xCA, 0xFF, 0xC6, 0xAC, 0x45, 0x42, 0xDE, 0x31 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x8D, 0xD4, 0x5A, 0x2D, 0xDF, 0x90, 0x79, 0x6C },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x10, 0x29, 0xD5, 0x5E, 0x88, 0x0E, 0xC2, 0xD0 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x5D, 0x86, 0xCB, 0x23, 0x63, 0x9D, 0xBE, 0xA9 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x1D, 0x1C, 0xA8, 0x53, 0xAE, 0x7C, 0x0C, 0x5F },
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xCE, 0x33, 0x23, 0x29, 0x24, 0x8F, 0x32, 0x28 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x84, 0x05, 0xD1, 0xAB, 0xE2, 0x4F, 0xB9, 0x42 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xE6, 0x43, 0xD7, 0x80, 0x90, 0xCA, 0x42, 0x07 },
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x48, 0x22, 0x1B, 0x99, 0x37, 0x74, 0x8A, 0x23 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xDD, 0x7C, 0x0B, 0xBD, 0x61, 0xFA, 0xFD, 0x54 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x2F, 0xBC, 0x29, 0x1A, 0x57, 0x0D, 0xB5, 0xC4 },
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xE0, 0x7C, 0x30, 0xD7, 0xE4, 0xE2, 0x6E, 0x12 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x09, 0x53, 0xE2, 0x25, 0x8E, 0x8E, 0x90, 0xA1 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x5B, 0x71, 0x1B, 0xC4, 0xCE, 0xEB, 0xF2, 0xEE },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xCC, 0x08, 0x3F, 0x1E, 0x6D, 0x9E, 0x85, 0xF6 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x08 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xD2, 0xFD, 0x88, 0x67, 0xD5, 0x0D, 0x2D, 0xFE },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x06, 0xE7, 0xEA, 0x22, 0xCE, 0x92, 0x70, 0x8F },
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x16, 0x6B, 0x40, 0xB4, 0x4A, 0xBA, 0x4B, 0xD6 },
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01 } }
	};

/* Inverse Permutation and Expansion test: Encrypt */

DES_TEST des56_ecb_vectors_vp[DES56_TEST_COUNT_VP] = {
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
	  { 0x95, 0xF8, 0xA5, 0xE5, 0xDD, 0x31, 0xD9, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
	  { 0xDD, 0x7F, 0x12, 0x1C, 0xA5, 0x01, 0x56, 0x19 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
	  { 0x2E, 0x86, 0x53, 0x10, 0x4F, 0x38, 0x34, 0xEA } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x4B, 0xD3, 0x88, 0xFF, 0x6C, 0xD8, 0x1D, 0x4F } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x20, 0xB9, 0xE7, 0x67, 0xB2, 0xFB, 0x14, 0x56 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
	  { 0x55, 0x57, 0x93, 0x80, 0xD7, 0x71, 0x38, 0xEF } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x6C, 0xC5, 0xDE, 0xFA, 0xAF, 0x04, 0x51, 0x2F } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x0D, 0x9F, 0x27, 0x9B, 0xA5, 0xD8, 0x72, 0x60 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xD9, 0x03, 0x1B, 0x02, 0x71, 0xBD, 0x5A, 0x0A } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x42, 0x42, 0x50, 0xB3, 0x7C, 0x3D, 0xD9, 0x51 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xB8, 0x06, 0x1B, 0x7E, 0xCD, 0x9A, 0x21, 0xE5 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xF1, 0x5D, 0x0F, 0x28, 0x6B, 0x65, 0xBD, 0x28 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xAD, 0xD0, 0xCC, 0x8D, 0x6E, 0x5D, 0xEB, 0xA1 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xE6, 0xD5, 0xF8, 0x27, 0x52, 0xAD, 0x63, 0xD1 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xEC, 0xBF, 0xE3, 0xBD, 0x3F, 0x59, 0x1A, 0x5E } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xF3, 0x56, 0x83, 0x43, 0x79, 0xD1, 0x65, 0xCD } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x2B, 0x9F, 0x98, 0x2F, 0x20, 0x03, 0x7F, 0xA9 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x40, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x88, 0x9D, 0xE0, 0x68, 0xA1, 0x6F, 0x0B, 0xE6 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x20, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xE1, 0x9E, 0x27, 0x5D, 0x84, 0x6A, 0x12, 0x98 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x32, 0x9A, 0x8E, 0xD5, 0x23, 0xD7, 0x1A, 0xEC } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xE7, 0xFC, 0xE2, 0x25, 0x57, 0xD2, 0x3C, 0x97 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x12, 0xA9, 0xF5, 0x81, 0x7F, 0xF2, 0xD6, 0x5D } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00, 0x00 },
	  { 0xA4, 0x84, 0xC3, 0xAD, 0x38, 0xDC, 0x9C, 0x19 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xFB, 0xE0, 0x0A, 0x8A, 0x1E, 0xF8, 0xAD, 0x72 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x75, 0x0D, 0x07, 0x94, 0x07, 0x52, 0x13, 0x63 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x40, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x64, 0xFE, 0xED, 0x9C, 0x72, 0x4C, 0x2F, 0xAF } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x20, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xF0, 0x2B, 0x26, 0x3B, 0x32, 0x8E, 0x2B, 0x60 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x9D, 0x64, 0x55, 0x5A, 0x9A, 0x10, 0xB8, 0x52 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xD1, 0x06, 0xFF, 0x0B, 0xED, 0x52, 0x55, 0xD7 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xE1, 0x65, 0x2C, 0x6B, 0x13, 0x8C, 0x64, 0xA5 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xE4, 0x28, 0x58, 0x11, 0x86, 0xEC, 0x8F, 0x46 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xAE, 0xB5, 0xF5, 0xED, 0xE2, 0x2D, 0x1A, 0x36 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00, 0x00 },  
	  { 0xE9, 0x43, 0xD7, 0x56, 0x8A, 0xEC, 0x0C, 0x5C } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x40, 0x00, 0x00, 0x00 },  
	  { 0xDF, 0x98, 0xC8, 0x27, 0x6F, 0x54, 0xB0, 0x4B } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x20, 0x00, 0x00, 0x00 },  
	  { 0xB1, 0x60, 0xE4, 0x68, 0x0F, 0x6C, 0x69, 0x6F } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00, 0x00 },  
	  { 0xFA, 0x07, 0x52, 0xB0, 0x7D, 0x9C, 0x4A, 0xB8 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00, 0x00 },  
	  { 0xCA, 0x3A, 0x2B, 0x03, 0x6D, 0xBC, 0x85, 0x02 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00, 0x00 },  
	  { 0x5E, 0x09, 0x05, 0x51, 0x7B, 0xB5, 0x9B, 0xCF } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00 },  
	  { 0x81, 0x4E, 0xEB, 0x3B, 0x91, 0xD9, 0x07, 0x26 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00 },
	  { 0x4D, 0x49, 0xDB, 0x15, 0x32, 0x91, 0x9C, 0x9F } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00, 0x00 },  
	  { 0x25, 0xEB, 0x5F, 0xC3, 0xF8, 0xCF, 0x06, 0x21 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x00, 0x00 },  
	  { 0xAB, 0x6A, 0x20, 0xC0, 0x62, 0x0D, 0x1C, 0x6F } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x00, 0x00 },  
	  { 0x79, 0xE9, 0x0D, 0xBC, 0x98, 0xF9, 0x2C, 0xCA } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00, 0x00 },  
	  { 0x86, 0x6E, 0xCE, 0xDD, 0x80, 0x72, 0xBB, 0x0E } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x00, 0x00 },  
	  { 0x8B, 0x54, 0x53, 0x6F, 0x2F, 0x3E, 0x64, 0xA8 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00, 0x00 },  
	  { 0xEA, 0x51, 0xD3, 0x97, 0x55, 0x95, 0xB8, 0x6B } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00 },  
	  { 0xCA, 0xFF, 0xC6, 0xAC, 0x45, 0x42, 0xDE, 0x31 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00 },  
	  { 0x8D, 0xD4, 0x5A, 0x2D, 0xDF, 0x90, 0x79, 0x6C } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80, 0x00 },  
	  { 0x10, 0x29, 0xD5, 0x5E, 0x88, 0x0E, 0xC2, 0xD0 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40, 0x00 },  
	  { 0x5D, 0x86, 0xCB, 0x23, 0x63, 0x9D, 0xBE, 0xA9 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x00 },  
	  { 0x1D, 0x1C, 0xA8, 0x53, 0xAE, 0x7C, 0x0C, 0x5F } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10, 0x00 },  
	  { 0xCE, 0x33, 0x23, 0x29, 0x24, 0x8F, 0x32, 0x28 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x08, 0x00 },  
	  { 0x84, 0x05, 0xD1, 0xAB, 0xE2, 0x4F, 0xB9, 0x42 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04, 0x00 },  
	  { 0xE6, 0x43, 0xD7, 0x80, 0x90, 0xCA, 0x42, 0x07 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00 },  
	  { 0x48, 0x22, 0x1B, 0x99, 0x37, 0x74, 0x8A, 0x23 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01, 0x00 },  
	  { 0xDD, 0x7C, 0x0B, 0xBD, 0x61, 0xFA, 0xFD, 0x54 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x80 },
	  { 0x2F, 0xBC, 0x29, 0x1A, 0x57, 0x0D, 0xB5, 0xC4 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x40 },  
	  { 0xE0, 0x7C, 0x30, 0xD7, 0xE4, 0xE2, 0x6E, 0x12 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x20 },  
	  { 0x09, 0x53, 0xE2, 0x25, 0x8E, 0x8E, 0x90, 0xA1 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x10 },  
	  { 0x5B, 0x71, 0x1B, 0xC4, 0xCE, 0xEB, 0xF2, 0xEE } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x08 },  
	  { 0xCC, 0x08, 0x3F, 0x1E, 0x6D, 0x9E, 0x85, 0xF6 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x04 },  
	  { 0xD2, 0xFD, 0x88, 0x67, 0xD5, 0x0D, 0x2D, 0xFE } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02 },  
	  { 0x06, 0xE7, 0xEA, 0x22, 0xCE, 0x92, 0x70, 0x8F } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01 },  
	  { 0x16, 0x6B, 0x40, 0xB4, 0x4A, 0xBA, 0x4B, 0xD6 } }
	}; 

/* Key Permutation tests: Encrypt */

DES_TEST des56_ecb_vectors_kp[DES56_TEST_COUNT_KP] = {
	{ { 0x80, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
	  { 0x95, 0xA8, 0xD7, 0x28, 0x13, 0xDA, 0xA9, 0x4D } },
	{ { 0x40, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
	  { 0x0E, 0xEC, 0x14, 0x87, 0xDD, 0x8C, 0x26, 0xD5 } },
	{ { 0x20, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
	  { 0x7A, 0xD1, 0x6F, 0xFB, 0x79, 0xC4, 0x59, 0x26 } },
	{ { 0x10, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xD3, 0x74, 0x62, 0x94, 0xCA, 0x6A, 0x6C, 0xF3 } },
	{ { 0x08, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x80, 0x9F, 0x5F, 0x87, 0x3C, 0x1F, 0xD7, 0x61 } },
	{ { 0x04, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xC0, 0x2F, 0xAF, 0xFE, 0xC9, 0x89, 0xD1, 0xFC } },
	{ { 0x02, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x46, 0x15, 0xAA, 0x1D, 0x33, 0xE7, 0x2F, 0x10 } },
	{ { 0x01, 0x80, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x20, 0x55, 0x12, 0x33, 0x50, 0xC0, 0x08, 0x58 } },
	{ { 0x01, 0x40, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xDF, 0x3B, 0x99, 0xD6, 0x57, 0x73, 0x97, 0xC8 } },
	{ { 0x01, 0x20, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x31, 0xFE, 0x17, 0x36, 0x9B, 0x52, 0x88, 0xC9 } },
	{ { 0x01, 0x10, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xDF, 0xDD, 0x3C, 0xC6, 0x4D, 0xAE, 0x16, 0x42 } },
	{ { 0x01, 0x08, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
	  { 0x17, 0x8C, 0x83, 0xCE, 0x2B, 0x39, 0x9D, 0x94 } },
	{ { 0x01, 0x04, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x50, 0xF6, 0x36, 0x32, 0x4A, 0x9B, 0x7F, 0x80 } },
	{ { 0x01, 0x02, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xA8, 0x46, 0x8E, 0xE3, 0xBC, 0x18, 0xF0, 0x6D } },
	{ { 0x01, 0x01, 0x80, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xA2, 0xDC, 0x9E, 0x92, 0xFD, 0x3C, 0xDE, 0x92 } },
	{ { 0x01, 0x01, 0x40, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xCA, 0xC0, 0x9F, 0x79, 0x7D, 0x03, 0x12, 0x87 } },
	{ { 0x01, 0x01, 0x20, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x90, 0xBA, 0x68, 0x0B, 0x22, 0xAE, 0xB5, 0x25 } },
	{ { 0x01, 0x01, 0x10, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xCE, 0x7A, 0x24, 0xF3, 0x50, 0xE2, 0x80, 0xB6 } },
	{ { 0x01, 0x01, 0x08, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x88, 0x2B, 0xFF, 0x0A, 0xA0, 0x1A, 0x0B, 0x87 } },
	{ { 0x01, 0x01, 0x04, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x25, 0x61, 0x02, 0x88, 0x92, 0x45, 0x11, 0xC2 } },
	{ { 0x01, 0x01, 0x02, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xC7, 0x15, 0x16, 0xC2, 0x9C, 0x75, 0xD1, 0x70 } },
	{ { 0x01, 0x01, 0x01, 0x80, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x51, 0x99, 0xC2, 0x9A, 0x52, 0xC9, 0xF0, 0x59 } },
	{ { 0x01, 0x01, 0x01, 0x40, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xC2, 0x2F, 0x0A, 0x29, 0x4A, 0x71, 0xF2, 0x9F } },
	{ { 0x01, 0x01, 0x01, 0x20, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xEE, 0x37, 0x14, 0x83, 0x71, 0x4C, 0x02, 0xEA } },
	{ { 0x01, 0x01, 0x01, 0x10, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xA8, 0x1F, 0xBD, 0x44, 0x8F, 0x9E, 0x52, 0x2F } },
	{ { 0x01, 0x01, 0x01, 0x08, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x4F, 0x64, 0x4C, 0x92, 0xE1, 0x92, 0xDF, 0xED } },
	{ { 0x01, 0x01, 0x01, 0x04, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x1A, 0xFA, 0x9A, 0x66, 0xA6, 0xDF, 0x92, 0xAE } },
	{ { 0x01, 0x01, 0x01, 0x02, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xB3, 0xC1, 0xCC, 0x71, 0x5C, 0xB8, 0x79, 0xD8 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x80, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
	  { 0x19, 0xD0, 0x32, 0xE6, 0x4A, 0xB0, 0xBD, 0x8B } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x40, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x3C, 0xFA, 0xA7, 0xA7, 0xDC, 0x87, 0x20, 0xDC } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x20, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xB7, 0x26, 0x5F, 0x7F, 0x44, 0x7A, 0xC6, 0xF3 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x10, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x9D, 0xB7, 0x3B, 0x3C, 0x0D, 0x16, 0x3F, 0x54 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x08, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x81, 0x81, 0xB6, 0x5B, 0xAB, 0xF4, 0xA9, 0x75 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x04, 0x01, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x93, 0xC9, 0xB6, 0x40, 0x42, 0xEA, 0xA2, 0x40 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x02, 0x01, 0x01, 0x01 },
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x55, 0x70, 0x53, 0x08, 0x29, 0x70, 0x55, 0x92 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x80, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x86, 0x38, 0x80, 0x9E, 0x87, 0x87, 0x87, 0xA0 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x40, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x41, 0xB9, 0xA7, 0x9A, 0xF7, 0x9A, 0xC2, 0x08 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x20, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x7A, 0x9B, 0xE4, 0x2F, 0x20, 0x09, 0xA8, 0x92 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x10, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x29, 0x03, 0x8D, 0x56, 0xBA, 0x6D, 0x27, 0x45 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x08, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x54, 0x95, 0xC6, 0xAB, 0xF1, 0xE5, 0xDF, 0x51 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x04, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xAE, 0x13, 0xDB, 0xD5, 0x61, 0x48, 0x89, 0x33 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x02, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x02, 0x4D, 0x1F, 0xFA, 0x89, 0x04, 0xE3, 0x89 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x80, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xD1, 0x39, 0x97, 0x12, 0xF9, 0x9B, 0xF0, 0x2E } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x40, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x14, 0xC1, 0xD7, 0xC1, 0xCF, 0xFE, 0xC7, 0x9E } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x20, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x1D, 0xE5, 0x27, 0x9D, 0xAE, 0x3B, 0xED, 0x6F } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x10, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
	  { 0xE9, 0x41, 0xA3, 0x3F, 0x85, 0x50, 0x13, 0x03 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x08, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xDA, 0x99, 0xDB, 0xBC, 0x9A, 0x03, 0xF3, 0x79 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x04, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xB7, 0xFC, 0x92, 0xF9, 0x1D, 0x8E, 0x92, 0xE9 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x02, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xAE, 0x8E, 0x5C, 0xAA, 0x3C, 0xA0, 0x4E, 0x85 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x80 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x9C, 0xC6, 0x2D, 0xF4, 0x3B, 0x6E, 0xED, 0x74 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x40 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xD8, 0x63, 0xDB, 0xB5, 0xC5, 0x9A, 0x91, 0xA0 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x20 },
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xA1, 0xAB, 0x21, 0x90, 0x54, 0x5B, 0x91, 0xD7 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x10 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x08, 0x75, 0x04, 0x1E, 0x64, 0xC5, 0x70, 0xF7 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x08 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x5A, 0x59, 0x45, 0x28, 0xBE, 0xBE, 0xF1, 0xCC } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x04 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xFC, 0xDB, 0x32, 0x91, 0xDE, 0x21, 0xF0, 0xC0 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x02 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x86, 0x9E, 0xFD, 0x7F, 0x9F, 0x26, 0x5A, 0x09 } }
	}; 

/* Test of right-shifts in Decryption: Decrypt */

DES_TEST des56_ecb_vectors_rs[DES56_TEST_COUNT_RS] = {
	{ { 0x80, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x95, 0xA8, 0xD7, 0x28, 0x13, 0xDA, 0xA9, 0x4D },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x40, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x0E, 0xEC, 0x14, 0x87, 0xDD, 0x8C, 0x26, 0xD5 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x20, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x7A, 0xD1, 0x6F, 0xFB, 0x79, 0xC4, 0x59, 0x26 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x10, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xD3, 0x74, 0x62, 0x94, 0xCA, 0x6A, 0x6C, 0xF3 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x08, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x80, 0x9F, 0x5F, 0x87, 0x3C, 0x1F, 0xD7, 0x61 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x04, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0xC0, 0x2F, 0xAF, 0xFE, 0xC9, 0x89, 0xD1, 0xFC },
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x02, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x46, 0x15, 0xAA, 0x1D, 0x33, 0xE7, 0x2F, 0x10 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x80, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x20, 0x55, 0x12, 0x33, 0x50, 0xC0, 0x08, 0x58 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x40, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xDF, 0x3B, 0x99, 0xD6, 0x57, 0x73, 0x97, 0xC8 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x20, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x31, 0xFE, 0x17, 0x36, 0x9B, 0x52, 0x88, 0xC9 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x10, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xDF, 0xDD, 0x3C, 0xC6, 0x4D, 0xAE, 0x16, 0x42 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x08, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },
	  { 0x17, 0x8C, 0x83, 0xCE, 0x2B, 0x39, 0x9D, 0x94 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x04, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x50, 0xF6, 0x36, 0x32, 0x4A, 0x9B, 0x7F, 0x80 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x02, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xA8, 0x46, 0x8E, 0xE3, 0xBC, 0x18, 0xF0, 0x6D },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x80, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xA2, 0xDC, 0x9E, 0x92, 0xFD, 0x3C, 0xDE, 0x92 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x40, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xCA, 0xC0, 0x9F, 0x79, 0x7D, 0x03, 0x12, 0x87 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x20, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x90, 0xBA, 0x68, 0x0B, 0x22, 0xAE, 0xB5, 0x25 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x10, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xCE, 0x7A, 0x24, 0xF3, 0x50, 0xE2, 0x80, 0xB6 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x08, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x88, 0x2B, 0xFF, 0x0A, 0xA0, 0x1A, 0x0B, 0x87 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x04, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x25, 0x61, 0x02, 0x88, 0x92, 0x45, 0x11, 0xC2 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x02, 0x01, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xC7, 0x15, 0x16, 0xC2, 0x9C, 0x75, 0xD1, 0x70 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x80, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x51, 0x99, 0xC2, 0x9A, 0x52, 0xC9, 0xF0, 0x59 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x40, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xC2, 0x2F, 0x0A, 0x29, 0x4A, 0x71, 0xF2, 0x9F },
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x20, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xEE, 0x37, 0x14, 0x83, 0x71, 0x4C, 0x02, 0xEA },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x10, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xA8, 0x1F, 0xBD, 0x44, 0x8F, 0x9E, 0x52, 0x2F },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x08, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x4F, 0x64, 0x4C, 0x92, 0xE1, 0x92, 0xDF, 0xED },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x04, 0x01, 0x01, 0x01, 0x01 },  
	  { 0x1A, 0xFA, 0x9A, 0x66, 0xA6, 0xDF, 0x92, 0xAE },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x02, 0x01, 0x01, 0x01, 0x01 },  
	  { 0xB3, 0xC1, 0xCC, 0x71, 0x5C, 0xB8, 0x79, 0xD8 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x80, 0x01, 0x01, 0x01 },
	  { 0x19, 0xD0, 0x32, 0xE6, 0x4A, 0xB0, 0xBD, 0x8B },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x40, 0x01, 0x01, 0x01 },  
	  { 0x3C, 0xFA, 0xA7, 0xA7, 0xDC, 0x87, 0x20, 0xDC },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x20, 0x01, 0x01, 0x01 },  
	  { 0xB7, 0x26, 0x5F, 0x7F, 0x44, 0x7A, 0xC6, 0xF3 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x10, 0x01, 0x01, 0x01 },  
	  { 0x9D, 0xB7, 0x3B, 0x3C, 0x0D, 0x16, 0x3F, 0x54 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x08, 0x01, 0x01, 0x01 },  
	  { 0x81, 0x81, 0xB6, 0x5B, 0xAB, 0xF4, 0xA9, 0x75 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x04, 0x01, 0x01, 0x01 },  
	  { 0x93, 0xC9, 0xB6, 0x40, 0x42, 0xEA, 0xA2, 0x40 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x02, 0x01, 0x01, 0x01 },  
	  { 0x55, 0x70, 0x53, 0x08, 0x29, 0x70, 0x55, 0x92 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x80, 0x01, 0x01 },  
	  { 0x86, 0x38, 0x80, 0x9E, 0x87, 0x87, 0x87, 0xA0 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x40, 0x01, 0x01 },  
	  { 0x41, 0xB9, 0xA7, 0x9A, 0xF7, 0x9A, 0xC2, 0x08 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x20, 0x01, 0x01 },  
	  { 0x7A, 0x9B, 0xE4, 0x2F, 0x20, 0x09, 0xA8, 0x92 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x10, 0x01, 0x01 },  
	  { 0x29, 0x03, 0x8D, 0x56, 0xBA, 0x6D, 0x27, 0x45 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x08, 0x01, 0x01 },  
	  { 0x54, 0x95, 0xC6, 0xAB, 0xF1, 0xE5, 0xDF, 0x51 },
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x04, 0x01, 0x01 },  
	  { 0xAE, 0x13, 0xDB, 0xD5, 0x61, 0x48, 0x89, 0x33 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x02, 0x01, 0x01 },  
	  { 0x02, 0x4D, 0x1F, 0xFA, 0x89, 0x04, 0xE3, 0x89 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x80, 0x01 },  
	  { 0xD1, 0x39, 0x97, 0x12, 0xF9, 0x9B, 0xF0, 0x2E },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x40, 0x01 },  
	  { 0x14, 0xC1, 0xD7, 0xC1, 0xCF, 0xFE, 0xC7, 0x9E },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x20, 0x01 },  
	  { 0x1D, 0xE5, 0x27, 0x9D, 0xAE, 0x3B, 0xED, 0x6F },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x10, 0x01 },
	  { 0xE9, 0x41, 0xA3, 0x3F, 0x85, 0x50, 0x13, 0x03 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x08, 0x01 },  
	  { 0xDA, 0x99, 0xDB, 0xBC, 0x9A, 0x03, 0xF3, 0x79 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x04, 0x01 },  
	  { 0xB7, 0xFC, 0x92, 0xF9, 0x1D, 0x8E, 0x92, 0xE9 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x02, 0x01 },  
	  { 0xAE, 0x8E, 0x5C, 0xAA, 0x3C, 0xA0, 0x4E, 0x85 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x80 },  
	  { 0x9C, 0xC6, 0x2D, 0xF4, 0x3B, 0x6E, 0xED, 0x74 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x40 },  
	  { 0xD8, 0x63, 0xDB, 0xB5, 0xC5, 0x9A, 0x91, 0xA0 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x20 },  
	  { 0xA1, 0xAB, 0x21, 0x90, 0x54, 0x5B, 0x91, 0xD7 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x10 },  
	  { 0x08, 0x75, 0x04, 0x1E, 0x64, 0xC5, 0x70, 0xF7 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x08 },  
	  { 0x5A, 0x59, 0x45, 0x28, 0xBE, 0xBE, 0xF1, 0xCC },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x04 },  
	  { 0xFC, 0xDB, 0x32, 0x91, 0xDE, 0x21, 0xF0, 0xC0 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } },
	{ { 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x01, 0x02 },  
	  { 0x86, 0x9E, 0xFD, 0x7F, 0x9F, 0x26, 0x5A, 0x09 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 } }
	}; 

/* Data permutation test: Encrypt */

DES_TEST des56_ecb_vectors_dp[DES56_TEST_COUNT_DP] = {
	{ { 0x10, 0x46, 0x91, 0x34, 0x89, 0x98, 0x01, 0x31 },
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x88, 0xD5, 0x5E, 0x54, 0xF5, 0x4C, 0x97, 0xB4 } },
	{ { 0x10, 0x07, 0x10, 0x34, 0x89, 0x98, 0x80, 0x20 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x0C, 0x0C, 0xC0, 0x0C, 0x83, 0xEA, 0x48, 0xFD } },
	{ { 0x10, 0x07, 0x10, 0x34, 0xC8, 0x98, 0x01, 0x20 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x83, 0xBC, 0x8E, 0xF3, 0xA6, 0x57, 0x01, 0x83 } },
	{ { 0x10, 0x46, 0x10, 0x34, 0x89, 0x98, 0x80, 0x20 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xDF, 0x72, 0x5D, 0xCA, 0xD9, 0x4E, 0xA2, 0xE9 } },
	{ { 0x10, 0x86, 0x91, 0x15, 0x19, 0x19, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xE6, 0x52, 0xB5, 0x3B, 0x55, 0x0B, 0xE8, 0xB0 } },
	{ { 0x10, 0x86, 0x91, 0x15, 0x19, 0x58, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
	  { 0xAF, 0x52, 0x71, 0x20, 0xC4, 0x85, 0xCB, 0xB0 } },
	{ { 0x51, 0x07, 0xB0, 0x15, 0x19, 0x58, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x0F, 0x04, 0xCE, 0x39, 0x3D, 0xB9, 0x26, 0xD5 } },
	{ { 0x10, 0x07, 0xB0, 0x15, 0x19, 0x19, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xC9, 0xF0, 0x0F, 0xFC, 0x74, 0x07, 0x90, 0x67 } },
	{ { 0x31, 0x07, 0x91, 0x54, 0x98, 0x08, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x7C, 0xFD, 0x82, 0xA5, 0x93, 0x25, 0x2B, 0x4E } },
	{ { 0x31, 0x07, 0x91, 0x94, 0x98, 0x08, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xCB, 0x49, 0xA2, 0xF9, 0xE9, 0x13, 0x63, 0xE3 } },
	{ { 0x10, 0x07, 0x91, 0x15, 0xB9, 0x08, 0x01, 0x40 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x00, 0xB5, 0x88, 0xBE, 0x70, 0xD2, 0x3F, 0x56 } },
	{ { 0x31, 0x07, 0x91, 0x15, 0x98, 0x08, 0x01, 0x40 },
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x40, 0x6A, 0x9A, 0x6A, 0xB4, 0x33, 0x99, 0xAE } },
	{ { 0x10, 0x07, 0xD0, 0x15, 0x89, 0x98, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x6C, 0xB7, 0x73, 0x61, 0x1D, 0xCA, 0x9A, 0xDA } },
	{ { 0x91, 0x07, 0x91, 0x15, 0x89, 0x98, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x67, 0xFD, 0x21, 0xC1, 0x7D, 0xBB, 0x5D, 0x70 } },
	{ { 0x91, 0x07, 0xD0, 0x15, 0x89, 0x19, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x95, 0x92, 0xCB, 0x41, 0x10, 0x43, 0x07, 0x87 } },
	{ { 0x10, 0x07, 0xD0, 0x15, 0x98, 0x98, 0x01, 0x20 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xA6, 0xB7, 0xFF, 0x68, 0xA3, 0x18, 0xDD, 0xD3 } },
	{ { 0x10, 0x07, 0x94, 0x04, 0x98, 0x19, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x4D, 0x10, 0x21, 0x96, 0xC9, 0x14, 0xCA, 0x16 } },
	{ { 0x01, 0x07, 0x91, 0x04, 0x91, 0x19, 0x04, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x2D, 0xFA, 0x9F, 0x45, 0x73, 0x59, 0x49, 0x65 } },
	{ { 0x01, 0x07, 0x91, 0x04, 0x91, 0x19, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xB4, 0x66, 0x04, 0x81, 0x6C, 0x0E, 0x07, 0x74 } },
	{ { 0x01, 0x07, 0x94, 0x04, 0x91, 0x19, 0x04, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x6E, 0x7E, 0x62, 0x21, 0xA4, 0xF3, 0x4E, 0x87 } },
	{ { 0x19, 0x07, 0x92, 0x10, 0x98, 0x1A, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xAA, 0x85, 0xE7, 0x46, 0x43, 0x23, 0x31, 0x99 } },
	{ { 0x10, 0x07, 0x91, 0x19, 0x98, 0x19, 0x08, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x2E, 0x5A, 0x19, 0xDB, 0x4D, 0x19, 0x62, 0xD6 } },
	{ { 0x10, 0x07, 0x91, 0x19, 0x98, 0x1A, 0x08, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },
	  { 0x23, 0xA8, 0x66, 0xA8, 0x09, 0xD3, 0x08, 0x94 } },
	{ { 0x10, 0x07, 0x92, 0x10, 0x98, 0x19, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xD8, 0x12, 0xD9, 0x61, 0xF0, 0x17, 0xD3, 0x20 } },
	{ { 0x10, 0x07, 0x91, 0x15, 0x98, 0x19, 0x01, 0x0B },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x05, 0x56, 0x05, 0x81, 0x6E, 0x58, 0x60, 0x8F } },
	{ { 0x10, 0x04, 0x80, 0x15, 0x98, 0x19, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xAB, 0xD8, 0x8E, 0x8B, 0x1B, 0x77, 0x16, 0xF1 } },
	{ { 0x10, 0x04, 0x80, 0x15, 0x98, 0x19, 0x01, 0x02 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x53, 0x7A, 0xC9, 0x5B, 0xE6, 0x9D, 0xA1, 0xE1 } },
	{ { 0x10, 0x04, 0x80, 0x15, 0x98, 0x19, 0x01, 0x08 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xAE, 0xD0, 0xF6, 0xAE, 0x3C, 0x25, 0xCD, 0xD8 } },
	{ { 0x10, 0x02, 0x91, 0x14, 0x98, 0x10, 0x01, 0x04 },
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xB3, 0xE3, 0x5A, 0x5E, 0xE5, 0x3E, 0x7B, 0x8D } },
	{ { 0x10, 0x02, 0x91, 0x15, 0x98, 0x19, 0x01, 0x04 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x61, 0xC7, 0x9C, 0x71, 0x92, 0x1A, 0x2E, 0xF8 } },
	{ { 0x10, 0x02, 0x91, 0x15, 0x98, 0x10, 0x02, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0xE2, 0xF5, 0x72, 0x8F, 0x09, 0x95, 0x01, 0x3C } },
	{ { 0x10, 0x02, 0x91, 0x16, 0x98, 0x10, 0x01, 0x01 },  
	  { 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00 },  
	  { 0x1A, 0xEA, 0xC3, 0x9A, 0x61, 0xF0, 0xA4, 0x64 } }
	}; 

/* S-Box test: Encrypt */

DES_TEST des56_ecb_vectors_sb[DES56_TEST_COUNT_SB] = {
	{ { 0x7C, 0xA1, 0x10, 0x45, 0x4A, 0x1A, 0x6E, 0x57 },
	  { 0x01, 0xA1, 0xD6, 0xD0, 0x39, 0x77, 0x67, 0x42 },
	  { 0x69, 0x0F, 0x5B, 0x0D, 0x9A, 0x26, 0x93, 0x9B } },
	{ { 0x01, 0x31, 0xD9, 0x61, 0x9D, 0xC1, 0x37, 0x6E },  
	  { 0x5C, 0xD5, 0x4C, 0xA8, 0x3D, 0xEF, 0x57, 0xDA },  
	  { 0x7A, 0x38, 0x9D, 0x10, 0x35, 0x4B, 0xD2, 0x71 } },
	{ { 0x07, 0xA1, 0x13, 0x3E, 0x4A, 0x0B, 0x26, 0x86 },  
	  { 0x02, 0x48, 0xD4, 0x38, 0x06, 0xF6, 0x71, 0x72 },  
	  { 0x86, 0x8E, 0xBB, 0x51, 0xCA, 0xB4, 0x59, 0x9A } },
	{ { 0x38, 0x49, 0x67, 0x4C, 0x26, 0x02, 0x31, 0x9E },  
	  { 0x51, 0x45, 0x4B, 0x58, 0x2D, 0xDF, 0x44, 0x0A },  
	  { 0x71, 0x78, 0x87, 0x6E, 0x01, 0xF1, 0x9B, 0x2A } },
	{ { 0x04, 0xB9, 0x15, 0xBA, 0x43, 0xFE, 0xB5, 0xB6 },  
	  { 0x42, 0xFD, 0x44, 0x30, 0x59, 0x57, 0x7F, 0xA2 },  
	  { 0xAF, 0x37, 0xFB, 0x42, 0x1F, 0x8C, 0x40, 0x95 } },
	{ { 0x01, 0x13, 0xB9, 0x70, 0xFD, 0x34, 0xF2, 0xCE },  
	  { 0x05, 0x9B, 0x5E, 0x08, 0x51, 0xCF, 0x14, 0x3A },
	  { 0x86, 0xA5, 0x60, 0xF1, 0x0E, 0xC6, 0xD8, 0x5B } },
	{ { 0x01, 0x70, 0xF1, 0x75, 0x46, 0x8F, 0xB5, 0xE6 },
	  { 0x07, 0x56, 0xD8, 0xE0, 0x77, 0x47, 0x61, 0xD2 },  
	  { 0x0C, 0xD3, 0xDA, 0x02, 0x00, 0x21, 0xDC, 0x09 } },
	{ { 0x43, 0x29, 0x7F, 0xAD, 0x38, 0xE3, 0x73, 0xFE },  
	  { 0x76, 0x25, 0x14, 0xB8, 0x29, 0xBF, 0x48, 0x6A },  
	  { 0xEA, 0x67, 0x6B, 0x2C, 0xB7, 0xDB, 0x2B, 0x7A } },
	{ { 0x07, 0xA7, 0x13, 0x70, 0x45, 0xDA, 0x2A, 0x16 },  
	  { 0x3B, 0xDD, 0x11, 0x90, 0x49, 0x37, 0x28, 0x02 },  
	  { 0xDF, 0xD6, 0x4A, 0x81, 0x5C, 0xAF, 0x1A, 0x0F } },
	{ { 0x04, 0x68, 0x91, 0x04, 0xC2, 0xFD, 0x3B, 0x2F },  
	  { 0x26, 0x95, 0x5F, 0x68, 0x35, 0xAF, 0x60, 0x9A },  
	  { 0x5C, 0x51, 0x3C, 0x9C, 0x48, 0x86, 0xC0, 0x88 } },
	{ { 0x37, 0xD0, 0x6B, 0xB5, 0x16, 0xCB, 0x75, 0x46 },  
	  { 0x16, 0x4D, 0x5E, 0x40, 0x4F, 0x27, 0x52, 0x32 },  
	  { 0x0A, 0x2A, 0xEE, 0xAE, 0x3F, 0xF4, 0xAB, 0x77 } },
	{ { 0x1F, 0x08, 0x26, 0x0D, 0x1A, 0xC2, 0x46, 0x5E },
	  { 0x6B, 0x05, 0x6E, 0x18, 0x75, 0x9F, 0x5C, 0xCA },  
	  { 0xEF, 0x1B, 0xF0, 0x3E, 0x5D, 0xFA, 0x57, 0x5A } },
	{ { 0x58, 0x40, 0x23, 0x64, 0x1A, 0xBA, 0x61, 0x76 },  
	  { 0x00, 0x4B, 0xD6, 0xEF, 0x09, 0x17, 0x60, 0x62 },  
	  { 0x88, 0xBF, 0x0D, 0xB6, 0xD7, 0x0D, 0xEE, 0x56 } },
	{ { 0x02, 0x58, 0x16, 0x16, 0x46, 0x29, 0xB0, 0x07 },  
	  { 0x48, 0x0D, 0x39, 0x00, 0x6E, 0xE7, 0x62, 0xF2 },  
	  { 0xA1, 0xF9, 0x91, 0x55, 0x41, 0x02, 0x0B, 0x56 } },
	{ { 0x49, 0x79, 0x3E, 0xBC, 0x79, 0xB3, 0x25, 0x8F },  
	  { 0x43, 0x75, 0x40, 0xC8, 0x69, 0x8F, 0x3C, 0xFA },  
	  { 0x6F, 0xBF, 0x1C, 0xAF, 0xCF, 0xFD, 0x05, 0x56 } },
	{ { 0x4F, 0xB0, 0x5E, 0x15, 0x15, 0xAB, 0x73, 0xA7 },  
	  { 0x07, 0x2D, 0x43, 0xA0, 0x77, 0x07, 0x52, 0x92 },  
	  { 0x2F, 0x22, 0xE4, 0x9B, 0xAB, 0x7C, 0xA1, 0xAC } },
	{ { 0x49, 0xE9, 0x5D, 0x6D, 0x4C, 0xA2, 0x29, 0xBF },
	  { 0x02, 0xFE, 0x55, 0x77, 0x81, 0x17, 0xF1, 0x2A },
	  { 0x5A, 0x6B, 0x61, 0x2C, 0xC2, 0x6C, 0xCE, 0x4A } },
	{ { 0x01, 0x83, 0x10, 0xDC, 0x40, 0x9B, 0x26, 0xD6 },
	  { 0x1D, 0x9D, 0x5C, 0x50, 0x18, 0xF7, 0x28, 0xC2 },
	  { 0x5F, 0x4C, 0x03, 0x8E, 0xD1, 0x2B, 0x2E, 0x41 } },
	{ { 0x1C, 0x58, 0x7F, 0x1C, 0x13, 0x92, 0x4F, 0xEF },
	  { 0x30, 0x55, 0x32, 0x28, 0x6D, 0x6F, 0x29, 0x5A },
	  { 0x63, 0xFA, 0xC0, 0xD0, 0x34, 0xD9, 0xF7, 0x93 } }
	};

/* Triple DES TECB (EDE) test vectors */

#define TRIPLEDES_TEST_COUNT 1

typedef struct {
	unsigned char key[24];
	unsigned char plaintext[8];
	unsigned char ciphertext[8];
	} TRIPLEDES_TEST;

TRIPLEDES_TEST tripledes_vectors[TRIPLEDES_TEST_COUNT] = {
0x01, 0x23, 0x45, 0x67, 0x89, 0xab, 0xcd, 0xef, 0xfe, 0xdc, 0xba, 0x98,
0x76, 0x54, 0x32, 0x10, 0x89, 0xab, 0xcd, 0xef, 0x01, 0x23, 0x45, 0x67,
                              
0x01, 0x23, 0x45, 0x67, 0x89, 0xab, 0xcd, 0xe7,

0xde, 0x0b, 0x7c, 0x06, 0xae, 0x5e, 0x0e, 0xd5
};

/* CAST-128 Test Vectors from RFC2144 */

#define CAST_TEST_COUNT 1

typedef struct {
	unsigned char key[16];
	unsigned char plaintext[8];
	unsigned char ciphertext[8];
	} CAST_TEST;


CAST_TEST cast_ecb_vectors[CAST_TEST_COUNT] = {
	"\x01\x23\x45\x67\x12\x34\x56\x78\x23\x45\x67\x89\x34\x56\x78\x9A",
	"\x01\x23\x45\x67\x89\xAB\xCD\xEF",
	"\x23\x8B\x4F\xE5\x84\x7E\x44\xB2"
	};

// AES ECB test vectors FIPS-197

#define AES_TEST_COUNT 1

typedef struct {
	unsigned char key[32];
	unsigned char plaintext[16];
	unsigned char ciphertext[16];
	} AES_TEST;

AES_TEST aes_ecb_vectors[AES_TEST_COUNT] = {
0x00,0x01,0x02,0x03,0x04,0x05,0x06,0x07,0x08,0x09,0x0a,0x0b,0x0c,0x0d,0x0e,0x0f,
0x10,0x11,0x12,0x13,0x14,0x15,0x16,0x17,0x18,0x19,0x1a,0x1b,0x1c,0x1d,0x1e,0x1f,

0x00,0x11,0x22,0x33,0x44,0x55,0x66,0x77,0x88,0x99,0xaa,0xbb,0xcc,0xdd,0xee,0xff,

0x8e,0xa2,0xb7,0xca,0x51,0x67,0x45,0xbf,0xea,0xfc,0x49,0x90,0x4b,0x49,0x60,0x89
};

// Serpent ECB test vectors

#define SERPENT_TEST_COUNT 1

typedef struct {
	unsigned char key[32];
	unsigned char plaintext[16];
	unsigned char ciphertext[16];
	} SERPENT_TEST;

SERPENT_TEST serpent_vectors[SERPENT_TEST_COUNT] = {
0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
0x10, 0x11, 0x12, 0x13, 0x14, 0x15, 0x16, 0x17, 0x18, 0x19, 0x1a, 0x1b, 0x1c, 0x1d, 0x1e, 0x1f,

0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
0xde, 0x26, 0x9f, 0xf8, 0x33, 0xe4, 0x32, 0xb8, 0x5b, 0x2e, 0x88, 0xd2, 0x70, 0x1c, 0xe7, 0x5c
};

// Twofish ECB test vectors

#define TWOFISH_TEST_COUNT 1

typedef struct {
	unsigned char key[32];
	unsigned char plaintext[16];
	unsigned char ciphertext[16];
	} TWOFISH_TEST;

TWOFISH_TEST twofish_vectors[TWOFISH_TEST_COUNT] = {
0xD4, 0x3B, 0xB7, 0x55, 0x6E, 0xA3, 0x2E, 0x46, 0xF2, 0xA2, 0x82, 0xB7, 0xD4, 0x5B, 0x4E, 0x0D,
0x57, 0xFF, 0x73, 0x9D, 0x4D, 0xC9, 0x2C, 0x1B, 0xD7, 0xFC, 0x01, 0x70, 0x0C, 0xC8, 0x21, 0x6F,

0x90, 0xAF, 0xE9, 0x1B, 0xB2, 0x88, 0x54, 0x4F, 0x2C, 0x32, 0xDC, 0x23, 0x9B, 0x26, 0x35, 0xE6,
0x6C, 0xB4, 0x56, 0x1C, 0x40, 0xBF, 0x0A, 0x97, 0x05, 0x93, 0x1C, 0xB6, 0xD4, 0x08, 0xE7, 0xFA
};

/* Test vectors from FIPS 198a, RFC 2104, RFC 2202, and other sources. */

char *hmac_sha1_test_keys[] =
{
	"\x50\x51\x52\x53\x54\x55\x56\x57\x58\x59\x5a\x5b\x5c\x5d\x5e\x5f\x60\x61\x62\x63\x64\x65\x66\x67\x68\x69\x6a\x6b\x6c\x6d\x6e\x6f\x70\x71\x72\x73\x74\x75\x76\x77\x78\x79\x7a\x7b\x7c\x7d\x7e\x7f\x80\x81\x82\x83\x84\x85\x86\x87\x88\x89\x8a\x8b\x8c\x8d\x8e\x8f\x90\x91\x92\x93\x94\x95\x96\x97\x98\x99\x9a\x9b\x9c\x9d\x9e\x9f\xa0\xa1\xa2\xa3\xa4\xa5\xa6\xa7\xa8\xa9\xaa\xab\xac\xad\xae\xaf\xb0\xb1\xb2\xb3",
	"\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b\x0b",
	"Jefe",
	"\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA\xAA"
};

char *hmac_sha1_test_data[] =
{
	"Sample #3",
	"Hi There",
	"what do ya want for nothing?",
	"\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD"
      "\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD"
	"\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD\xDD"
};

char *hmac_sha1_test_vectors[] =
{
	"\xbc\xf4\x1e\xab\x8b\xb2\xd8\x02\xf3\xd0\x5c\xaf\x7c\xb0\x92\xec\xf8\xd1\xa3\xaa",
	"\xb6\x17\x31\x86\x55\x05\x72\x64\xe2\x8b\xc0\xb6\xfb\x37\x8c\x8e\xf1\x46\xbe\x00",
	"\xef\xfc\xdf\x6a\xe5\xeb\x2f\xa2\xd2\x74\x16\xd5\xf1\x84\xdf\x9c\x25\x9a\x7c\x79",
	"\x12\x5d\x73\x42\xb9\xac\x11\xcd\x91\xa3\x9a\xf4\x8a\xa1\x7b\x4f\x63\xf1\x75\xd3"
};

char *hmac_ripemd160_test_keys[] =
{
	"\x00\x11\x22\x33\x44\x55\x66\x77\x88\x99\xaa\xbb\xcc\xdd\xee\xff\x01\x23\x45\x67",
	"\x01\x23\x45\x67\x89\xab\xcd\xef\xfe\xdc\xba\x98\x76\x54\x32\x10\x00\x11\x22\x33",
};

char *hmac_ripemd160_test_data[] =
{
	"message digest",
	"12345678901234567890123456789012345678901234567890123456789012345678901234567890",
};

char *hmac_ripemd160_test_vectors[] =
{
	"\xf8\x36\x62\xcc\x8d\x33\x9c\x22\x7e\x60\x0f\xcd\x63\x6c\x57\xd2\x57\x1b\x1c\x34",
	"\x85\xf1\x64\x70\x3e\x61\xa6\x31\x31\xbe\x7e\x45\x95\x8e\x07\x94\x12\x39\x04\xf9",
};

char *hmac_whirlpool_test_key =
{
	"\x00\x11\x22\x33\x44\x55\x66\x77\x88\x99\xAA\xBB\xCC\xDD\xEE\xFF\x01\x23\x45\x67\x89\xAB\xCD\xEF\x00\x11\x22\x33\x44\x55\x66\x77\x88\x99\xAA\xBB\xCC\xDD\xEE\xFF\x01\x23\x45\x67\x89\xAB\xCD\xEF\x00\x11\x22\x33\x44\x55\x66\x77\x88\x99\xAA\xBB\xCC\xDD\xEE\xFF"
};

char *hmac_whirlpool_test_data =
{
	"abcdbcdecdefdefgefghfghighijhijkijkljklmklmnlmnomnopnopq"
};

char *hmac_whirlpool_test_vectors =
{
	"\x03\x91\xd2\x80\x00\xb6\x62\xbb\xb8\xe6\x23\x3e\xe8\x6c\xf2\xb2\x84\x74\x4c\x73"
	"\x8b\x58\x00\xba\x28\x12\xed\x52\x6f\xe3\x15\x3a\xb1\xba\xe7\xe2\x36\xbe\x96\x54"
	"\x49\x3f\x19\xfa\xce\xa6\x44\x1f\x60\xf5\xf0\x18\x93\x09\x11\xa5\xe5\xce\xd8\xf2"
	"\x6a\xbf\xa4\x02"
};

unsigned char ks_tmp[MAX_EXPANDED_KEY]; 

void
CipherInit2(int cipher, void* key, void* ks, int key_len) 
{
	switch (cipher)
	{
	case BLOWFISH:
		BF_set_key (ks, key_len,  key); 
		break;

	case AES:
		CipherInit(cipher,key,ks);
		break;

	case DES56:
		CipherInit(cipher,key,ks);
		break;

	case CAST:
		CipherInit(cipher,key,ks);
		break;

	case SERPENT:
		CipherInit(cipher,key,ks);
		break;

	case TRIPLEDES:
		CipherInit(cipher,key,ks);
		break;

	case TWOFISH:
		CipherInit(cipher,key,ks);
		break;
	}
}

BOOL Des56TestLoop(void *test_vectors, int nVectorCount, int enc)
{
	char key[32];
	__int32 tmp[2];
	int i, cipher;

	cipher = DES56;
	for (i=0;i<nVectorCount;i++)
	{
		memcpy(key, ((DES_TEST*)test_vectors)->key, 8);
		memcpy(tmp, ((DES_TEST*)test_vectors)->plaintext, 8);

		memset(ks_tmp,0,sizeof(ks_tmp));
		CipherInit2(cipher, key, ks_tmp, 8);

		if (enc) 
		{
			EncipherBlock(cipher, tmp, ks_tmp);
		}
		else
		{
			DecipherBlock(cipher, tmp, ks_tmp);
		}

		if (memcmp(((DES_TEST*)test_vectors)->ciphertext, tmp,8)!=0)
			return FALSE;
	}

	return TRUE;
}


typedef struct {
	unsigned __int8 key1[32];
	unsigned __int8 key2[16];
	unsigned __int8 index[16];
	unsigned __int8 plaintext[16];
	unsigned __int8 ciphertext[16];
} LRW_TEST;

#define LRW_TEST_COUNT 2

LRW_TEST lrw_vectors[LRW_TEST_COUNT] = {
	{
		{ 0xf8, 0xd4, 0x76, 0xff, 0xd6, 0x46, 0xee, 0x6c, 0x23, 0x84, 0xcb, 0x1c, 0x77, 0xd6, 0x19, 0x5d,
		  0xfe, 0xf1, 0xa9, 0xf3, 0x7b, 0xbc, 0x8d, 0x21, 0xa7, 0x9c, 0x21, 0xf8, 0xcb, 0x90, 0x02, 0x89 },
		{ 0xa8, 0x45, 0x34, 0x8e, 0xc8, 0xc5, 0xb5, 0xf1, 0x26, 0xf5, 0x0e, 0x76, 0xfe, 0xfd, 0x1b, 0x1e },
		{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x01 },
		{ 0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46 },
		{ 0xbd, 0x06, 0xb8, 0xe1, 0xdb, 0x98, 0x89, 0x9e, 0xc4, 0x98, 0xe4, 0x91, 0xcf, 0x1c, 0x70, 0x2b }
	},
	{
		{ 0xfb, 0x76, 0x15, 0xb2, 0x3d, 0x80, 0x89, 0x1d, 0xd4, 0x70, 0x98, 0x0b, 0xc7, 0x95, 0x84, 0xc8,
		  0xb2, 0xfb, 0x64, 0xce, 0x60, 0x97, 0x87, 0x8d, 0x17, 0xfc, 0xe4, 0x5a, 0x49, 0xe8, 0x30, 0xb7 },
		{ 0x6e, 0x78, 0x17, 0xe7, 0x2d, 0x5e, 0x12, 0xd4, 0x60, 0x64, 0x04, 0x7a, 0xf1, 0x2f, 0x9e, 0x0c },
		{ 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x02, 0x00, 0x00, 0x00, 0x00 },
		{ 0x30, 0x31, 0x32, 0x33, 0x34, 0x35, 0x36, 0x37, 0x38, 0x39, 0x41, 0x42, 0x43, 0x44, 0x45, 0x46 },
		{ 0x5b, 0x90, 0x8e, 0xc1, 0xab, 0xdd, 0x67, 0x5f, 0x3d, 0x69, 0x8a, 0x95, 0x53, 0xc8, 0x9c, 0xe5 }
	}
};


BOOL LRWAesTest (PCRYPTO_INFO ci)
{
	unsigned __int8 p[16];
	int i;

	for (i = 0; i < LRW_TEST_COUNT; i++)
	{
		ci->ea = EAGetByName ("AES");
		if (ci->ea == 0)
			return TRUE;

		ci->mode = LRW;

		if (EAInit (ci->ea, lrw_vectors[i].key1, ci->ks) != 0)
			return FALSE;

		memcpy (&ci->iv, lrw_vectors[i].key2, sizeof (lrw_vectors[i].key2));
		if (!EAInitMode (ci))
			return FALSE;

		memcpy (p, lrw_vectors[i].plaintext, sizeof (p));

		EncryptBufferLRW128 (p, sizeof (p), BE64(((unsigned __int64 *)(lrw_vectors[i].index))[1]), ci);

		if (memcmp (lrw_vectors[i].ciphertext, p, sizeof (p)) != 0)
			return FALSE;
	}

	return TRUE;
}


BOOL TestSectorBufEncryption (PCRYPTO_INFO ci)
{
	unsigned char buf[SECTOR_SIZE * 2];
	unsigned int i;
	char name[64];
	unsigned __int32 crc;
	unsigned __int64 secNo = 0x0234567890ABCDEFull;
	int blockSize;
	BOOL lrw64InitDone = FALSE;
	BOOL lrw128InitDone = FALSE;

	for (i = 0; i < sizeof (buf); i++)
		buf[i] = (unsigned char) i;

	for (i = 0; i < sizeof (ci->iv); i++)
		ci->iv[i] = (unsigned char) i;

	// Test all EAs
	for (ci->ea = EAGetFirst (); ci->ea != 0; ci->ea = EAGetNext (ci->ea))
	{
		EAGetName (name, ci->ea);
		blockSize = CipherGetBlockSize (EAGetFirstCipher (ci->ea));

		if (EAInit (ci->ea, (unsigned char *)buf, ci->ks) == ERR_CIPHER_INIT_FAILURE)
			return FALSE;

		// Test all available modes of operation
		for (ci->mode = EAGetFirstMode (ci->ea);
			ci->mode != 0;
			ci->mode = EAGetNextMode (ci->ea, ci->mode))
		{
			if (ci->mode == LRW
				&& (blockSize == 8 && !lrw64InitDone || blockSize == 16 && !lrw128InitDone ))
			{
				if (!EAInitMode (ci))
					return FALSE;

				if (blockSize == 8)
					lrw64InitDone = TRUE;
				else if (blockSize == 16)
					lrw128InitDone = TRUE;
			}

			EncryptSectors ((unsigned __int32 *)buf, secNo, sizeof (buf) / SECTOR_SIZE, ci);
			crc = crc32 (buf, sizeof (buf));

			switch (ci->mode)
			{
			case LRW:
				if (strcmp (name, "AES") == 0 &&					crc != 0x5237acf9)
					return FALSE;
				if (strcmp (name, "Blowfish") == 0 &&				crc != 0xf94d5300)
					return FALSE;
				if (strcmp (name, "CAST5") == 0 &&					crc != 0x33971e82)
					return FALSE;
				if (strcmp (name, "Serpent") == 0 &&				crc != 0x7fb86805)
					return FALSE;
				if (strcmp (name, "Triple DES") == 0 &&				crc != 0x2b20bb84)
					return FALSE;
				if (strcmp (name, "Twofish") == 0 &&				crc != 0xa9de0f0b)
					return FALSE;
				if (strcmp (name, "AES-Twofish") == 0 &&			crc != 0x4ed0fd80)
					return FALSE;
				if (strcmp (name, "AES-Twofish-Serpent") == 0 &&	crc != 0xea04b3cf)
					return FALSE;
				if (strcmp (name, "Serpent-AES") == 0 &&			crc != 0x0d33596a)
					return FALSE;
				if (strcmp (name, "Serpent-Twofish-AES") == 0 &&	crc != 0x2845d0e3)
					return FALSE;
				if (strcmp (name, "Twofish-Serpent") == 0 &&		crc != 0xca65c5cd)
					return FALSE;
				break;

			case CBC:
			case INNER_CBC:
			case OUTER_CBC:
				if (strcmp (name, "AES") == 0 &&					crc != 0x2274f53d)
					return FALSE;
				if (strcmp (name, "Blowfish") == 0 &&				crc != 0x033899a1)
					return FALSE;
				if (strcmp (name, "CAST5") == 0 &&					crc != 0x331cecc7)
					return FALSE;
				if (strcmp (name, "Serpent") == 0 &&				crc != 0x42dff3d4)
					return FALSE;
				if (strcmp (name, "Triple DES") == 0 &&				crc != 0xfe497d0c)
					return FALSE;
				if (strcmp (name, "AES-Blowfish") == 0 &&			crc != 0xa7a80c84)
					return FALSE;
				if (strcmp (name, "AES-Blowfish-Serpent") == 0 &&	crc != 0xa0584562)
					return FALSE;
				if (strcmp (name, "AES-Twofish") == 0 &&			crc != 0x3c226444)
					return FALSE;
				if (strcmp (name, "AES-Twofish-Serpent") == 0 &&	crc != 0x5e5e77fd)
					return FALSE;
				if (strcmp (name, "Serpent-AES") == 0 &&			crc != 0x57c612d5)
					return FALSE;
				if (strcmp (name, "Serpent-Twofish-AES") == 0 &&	crc != 0x081e045a)
					return FALSE;
				if (strcmp (name, "Twofish-Serpent") == 0 &&		crc != 0xa7b659f3)
					return FALSE;
				break;
			}

			if (crc == 0xb70b4c26)
				return FALSE;

			DecryptSectors ((unsigned __int32 *)buf, secNo, sizeof (buf) / SECTOR_SIZE, ci);

			if (crc32 (buf, sizeof (buf)) != 0xb70b4c26)
				return FALSE;

			EncryptBuffer ((unsigned __int32 *)buf, sizeof (buf), ci);
			crc = crc32 (buf, sizeof (buf));

			switch (ci->mode)
			{
			case LRW:
				if (strcmp (name, "AES") == 0 &&					crc != 0x5ae1e3d8)
					return FALSE;
				if (strcmp (name, "Blowfish") == 0 &&				crc != 0x2738426f)
					return FALSE;
				if (strcmp (name, "AES-Twofish-Serpent") == 0 &&	crc != 0x14f2948a)
					return FALSE;
				break;

			case CBC:
			case INNER_CBC:
			case OUTER_CBC:
				if (strcmp (name, "AES") == 0 &&					crc != 0x960f740e)
					return FALSE;
				if (strcmp (name, "Blowfish") == 0 &&				crc != 0x7e1cfabb)
					return FALSE;
				if (strcmp (name, "CAST5") == 0 &&					crc != 0xeaae21c8)
					return FALSE;
				if (strcmp (name, "Serpent") == 0 &&				crc != 0xa8139d62)
					return FALSE;
				if (strcmp (name, "Triple DES") == 0 &&				crc != 0xecf5d7d0)
					return FALSE;
				if (strcmp (name, "AES-Blowfish") == 0 &&			crc != 0xb70171b6)
					return FALSE;
				if (strcmp (name, "AES-Blowfish-Serpent") == 0 &&	crc != 0x1e749a87)
					return FALSE;
				if (strcmp (name, "AES-Twofish") == 0 &&			crc != 0xb4b8bb9b)
					return FALSE;
				if (strcmp (name, "AES-Twofish-Serpent") == 0 &&	crc != 0x76b6c1cb)
					return FALSE;
				if (strcmp (name, "Serpent-AES") == 0 &&			crc != 0x634f12ed)
					return FALSE;
				if (strcmp (name, "Serpent-Twofish-AES") == 0 &&	crc != 0xe54bc1b9)
					return FALSE;
				if (strcmp (name, "Twofish-Serpent") == 0 &&		crc != 0x21cdb382)
					return FALSE;
				break;
			}

			if (crc == 0xb70b4c26)
				return FALSE;

			DecryptBuffer ((unsigned __int32 *)buf, sizeof (buf), ci);

			if (crc32 (buf, sizeof (buf)) != 0xb70b4c26)
				return FALSE;
		}
	}
	return TRUE;
}


BOOL AutoTestAlgorithms (void)
{
	PCRYPTO_INFO ci;
	char key[32];
	unsigned char tmp[16];
	BOOL bFailed = FALSE;
	int i;

	ci = crypto_open ();
	if (!ci)
		return FALSE;

	memset (ci, 0, sizeof (*ci));

	/* Blowfish */

	for (i=0;i<BF_TEST_COUNT;i++)
	{			
		memcpy(key, bf_ecb_vectors[i].key, 8);
		memcpy(tmp, bf_ecb_vectors[i].plaintext, 8);
		CipherInit2(BLOWFISH, key, ks_tmp, 8);
		BF_ecb_encrypt(tmp,tmp,(struct bf_key_st *)ks_tmp,1);
		if (memcmp(bf_ecb_vectors[i].ciphertext,tmp,8)!=0)
			break;
	}

	if (i==BF_TEST_COUNT)
	{
		/* Test the Blowfish code against Bruce Schneiers test vectors (1 & 2) and
		Mike Morgans test vector (3) */

		unsigned char *plain1 = ( unsigned char * ) "BLOWFISH";
		unsigned char *key1 = ( unsigned char * ) "abcdefghijklmnopqrstuvwxyz";
		unsigned char cipher1[] = { 0x32, 0x4E, 0xD0, 0xFE, 0xF4, 0x13, 0xA2, 0x03 };
		unsigned char plain2[] = { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 };
		unsigned char *key2 = ( unsigned char * ) "Who is John Galt?";
		unsigned char cipher2[] = { 0xCC, 0x91, 0x73, 0x2B, 0x80, 0x22, 0xF6, 0x84 };
		unsigned char plain3[] = { 0xFE, 0xDC, 0xBA, 0x98, 0x76, 0x54, 0x32, 0x10 };
		unsigned char key3[] = { 0x41, 0x79, 0x6E, 0xA0, 0x52, 0x61, 0x6E, 0xE4 };
		unsigned char cipher3[] = { 0xE1, 0x13, 0xF4, 0x10, 0x2C, 0xFC, 0xCE, 0x43 };
		unsigned char tmp[8];


		CipherInit2(BLOWFISH, key1, ks_tmp, 26);
		BF_ecb_encrypt(plain1,tmp,(struct bf_key_st *)ks_tmp,1);
		if (memcmp(cipher1,tmp,8)!=0)
			bFailed = TRUE;

		CipherInit2(BLOWFISH, key2, ks_tmp, 17);
		BF_ecb_encrypt(plain2,tmp,(struct bf_key_st *)ks_tmp,1);
		if (memcmp(cipher2,tmp,8)!=0)
			bFailed = TRUE;

		CipherInit2(BLOWFISH, key3, ks_tmp, 8);
		BF_ecb_encrypt(plain3,tmp,(struct bf_key_st *)ks_tmp,1);
		if (memcmp(cipher3,tmp,8)!=0)
			bFailed = TRUE;

	} else
		bFailed = TRUE;


	/* CAST5 */

	for (i=0;i<CAST_TEST_COUNT;i++)
	{			
		int cipher = CAST;
		memcpy(key, cast_ecb_vectors[i].key, 16);
		memcpy(tmp, cast_ecb_vectors[i].plaintext, 8);
		CipherInit2(cipher, key, ks_tmp, 16);
		EncipherBlock(cipher, tmp, ks_tmp);
		if (memcmp(cast_ecb_vectors[i].ciphertext, tmp,8)!=0)
			break;
	}

	if (i!=CAST_TEST_COUNT)
		bFailed = TRUE;


	/* DES */

	if (Des56TestLoop(des56_ecb_vectors_ip, DES56_TEST_COUNT_IP,1)!=TRUE)
		bFailed = TRUE;
	if (Des56TestLoop(des56_ecb_vectors_vp, DES56_TEST_COUNT_VP,1)!=TRUE)
		bFailed = TRUE;
	if (Des56TestLoop(des56_ecb_vectors_kp, DES56_TEST_COUNT_KP,1)!=TRUE)
		bFailed = TRUE;
	if (Des56TestLoop(des56_ecb_vectors_rs, DES56_TEST_COUNT_RS,0)!=TRUE)
		bFailed = TRUE;
	if (Des56TestLoop(des56_ecb_vectors_dp, DES56_TEST_COUNT_DP,1)!=TRUE)
		bFailed = TRUE;
	if (Des56TestLoop(des56_ecb_vectors_sb, DES56_TEST_COUNT_SB,1)!=TRUE)
		bFailed = TRUE;


	/* Triple DES (TECB, EDE) */

	for (i = 0; i < TRIPLEDES_TEST_COUNT; i++)
	{			
		int cipher = TRIPLEDES;
		memcpy(key, tripledes_vectors[i].key, sizeof(tripledes_vectors->key));
		memcpy(tmp, tripledes_vectors[i].plaintext, sizeof(tripledes_vectors->plaintext));
		CipherInit(cipher, key, ks_tmp);

		EncipherBlock(cipher, tmp, ks_tmp);
		if (memcmp(tripledes_vectors[i].ciphertext, tmp, sizeof(tripledes_vectors->ciphertext)) != 0)
			break;

		DecipherBlock(cipher, tmp, ks_tmp);
		if (memcmp(tripledes_vectors[i].plaintext, tmp, sizeof(tripledes_vectors->plaintext)) != 0)
			break;
	}
	if (i != TRIPLEDES_TEST_COUNT)
		bFailed = TRUE;


	/* AES */

	for (i = 0; i < AES_TEST_COUNT; i++)
	{			
		int cipher = AES;
		memcpy(key, aes_ecb_vectors[i].key, 32);
		memcpy(tmp, aes_ecb_vectors[i].plaintext, 16);
		CipherInit(cipher, key, ks_tmp);

		EncipherBlock(cipher, tmp, ks_tmp);
		if (memcmp(aes_ecb_vectors[i].ciphertext, tmp, 16) != 0)
			break;

		DecipherBlock(cipher, tmp, ks_tmp);
		if (memcmp(aes_ecb_vectors[i].plaintext, tmp, 16) != 0)
			break;
	}
	if (i != AES_TEST_COUNT)
		bFailed = TRUE;


	/* Serpent */

	for (i = 0; i < SERPENT_TEST_COUNT; i++)
	{			
		int cipher = SERPENT;
		memcpy(key, serpent_vectors[i].key, 32);
		memcpy(tmp, serpent_vectors[i].plaintext, 16);
		CipherInit(cipher, key, ks_tmp);

		EncipherBlock(cipher, tmp, ks_tmp);
		if (memcmp(serpent_vectors[i].ciphertext, tmp, 16) != 0)
			break;

		DecipherBlock(cipher, tmp, ks_tmp);
		if (memcmp(serpent_vectors[i].plaintext, tmp, 16) != 0)
			break;
	}
	if (i != SERPENT_TEST_COUNT)
		bFailed = TRUE;


	/* Twofish */

	for (i = 0; i < TWOFISH_TEST_COUNT; i++)
	{			
		int cipher = TWOFISH;
		memcpy(key, twofish_vectors[i].key, 32);
		memcpy(tmp, twofish_vectors[i].plaintext, 16);
		CipherInit(cipher, key, ks_tmp);

		EncipherBlock(cipher, tmp, ks_tmp);
		if (memcmp(twofish_vectors[i].ciphertext, tmp, 16) != 0)
			break;

		DecipherBlock(cipher, tmp, ks_tmp);
		if (memcmp(twofish_vectors[i].plaintext, tmp, 16) != 0)
			break;
	}
	if (i != TWOFISH_TEST_COUNT)
		bFailed = TRUE;

	
	/* PKCS #5 and HMACs */
#ifndef LINUX_DRIVER
	if (!test_pkcs5 ())
		bFailed = TRUE;
#endif

	/* CRC-32 */
	if (!crc32_selftests ())
		bFailed = TRUE;

	/* GF multiplicator */
#if 0
	if (!GfMulSelfTest ())
		bFailed = TRUE;
#endif

	/* LRW-AES */
	if (!LRWAesTest (ci))
		bFailed = TRUE;

	/* Sector and buffer algorithms */
	if (!TestSectorBufEncryption (ci))
		bFailed = TRUE;
	
	crypto_close (ci);
	return !bFailed;
}

#ifndef LINUX_DRIVER
BOOL test_hmac_sha1 ()
{
	int i;

	for (i = 0; i < 3; i++)
	{
		char digest[SHA1_DIGESTSIZE];
		hmac_sha1 (hmac_sha1_test_keys[i], strlen (hmac_sha1_test_keys[i]), hmac_sha1_test_data[i], strlen (hmac_sha1_test_data[i]), digest, SHA1_DIGESTSIZE);
		if (memcmp (digest, hmac_sha1_test_vectors[i], SHA1_DIGESTSIZE) != 0)
			return FALSE;
	}

	return TRUE;
}

BOOL test_hmac_ripemd160 ()
{
	unsigned int i;
	for (i = 0; i < sizeof (hmac_ripemd160_test_data) / sizeof(char *); i++)
	{
		char digest[RIPEMD160_DIGESTSIZE];
		hmac_ripemd160 (hmac_ripemd160_test_keys[i], RIPEMD160_DIGESTSIZE, hmac_ripemd160_test_data[i], strlen (hmac_ripemd160_test_data[i]), digest);
		if (memcmp (digest, hmac_ripemd160_test_vectors[i], RIPEMD160_DIGESTSIZE) != 0)
			return FALSE;
	}

	return TRUE;
}

BOOL test_hmac_whirlpool ()
{
	unsigned char digest[WHIRLPOOL_DIGESTSIZE];

	hmac_whirlpool (hmac_whirlpool_test_key, 64, hmac_whirlpool_test_data, strlen (hmac_whirlpool_test_data), digest, WHIRLPOOL_DIGESTSIZE);
	if (memcmp (digest, hmac_whirlpool_test_vectors, WHIRLPOOL_DIGESTSIZE) != 0)
		return FALSE;

	return TRUE;
}

BOOL test_pkcs5 ()
{
	char dk[4];

	/* HMAC-SHA-1 tests */
	if (test_hmac_sha1() == FALSE)
		return FALSE;

	/* HMAC-RIPEMD-160 tests */
	if (test_hmac_ripemd160() == FALSE)
		return FALSE;

	/* HMAC-Whirlpool tests */
	if (test_hmac_whirlpool() == FALSE)
		return FALSE;

	/* PKCS-5 test with HMAC-SHA-1 used as the PRF */
	derive_key_sha1 ("password", 8, "\x12\x34\x56\x78", 4, 5, dk, 4);
	if (memcmp (dk, "\x5c\x75\xce\xf0", 4) != 0)
		return FALSE;

	/* PKCS-5 test with HMAC-RIPEMD-160 used as the PRF */
	derive_key_ripemd160 ("password", 8, "\x12\x34\x56\x78", 4, 5, dk, 4);
	if (memcmp (dk, "\x7a\x3d\x7c\x03", 4) != 0)
		return FALSE;

	/* PKCS-5 test with HMAC-Whirlpool used as the PRF */
	derive_key_whirlpool ("password", 8, "\x12\x34\x56\x78", 4, 5, dk, 4);
	if (memcmp (dk, "\x50\x7c\x36\x6f", 4) != 0)
		return FALSE;

	return TRUE;
}
#endif /* LINUX_DRIVER */
 
